<?php

namespace Drupal\content_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\SubformStateInterface;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\content_block\QueryHelper;
use Drupal\content_block\LayoutRender\LayoutRender;
use Drupal\Core\Cache\Cache;
use Drupal\content_block\Plugin\ContentBlockOptionManager;

/**
 * Provides a dynamic content block for Layout Builder.
 *
 * @Block(
 *   id = "content_block",
 *   admin_label = @Translation("Content Block"),
 *   category = @Translation("Content")
 * )
 */
class ContentBlock extends BlockBase implements ContainerFactoryPluginInterface, TrustedCallbackInterface {

  /**
   * The dynamic query helper service.
   *
   * @var \Drupal\content_block\QueryHelper
   */
  protected $queryHelper;

  /**
   * The dynamic layout render service.
   *
   * @var \Drupal\content_block\LayoutRender\LayoutRender
   */
  protected $layoutRender;

  /**
   * The content block option manager service.
   *
   * @var \Drupal\content_block\Plugin\ContentBlockOptionManager
   */
  protected $contentBlockOptionManager;

  /**
   * Constructs a new instance.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, QueryHelper $queryHelper, LayoutRender $layoutRender, ContentBlockOptionManager $contentBlockOptionManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->queryHelper = $queryHelper;
    $this->layoutRender = $layoutRender;
    $this->contentBlockOptionManager = $contentBlockOptionManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('content_block.query_helper'),
      $container->get('content_block.layout_render'),
      $container->get('plugin.manager.content_block_option'),
    );
  }

  /**
   * AJAX callback to update dependent fields based on Entity Type.
   */
  public function updateDependentFields(array $form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);
    return $form['settings']['wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    // Get the complete form state.
    if ($form_state instanceof SubformStateInterface) {
      $form_state = $form_state->getCompleteFormState();
    }

    // Get the current configuration.
    $config = $this->getConfiguration();

    // Get the values from the configuration.
    $entity_type = $form_state->getValue('entity_type', $config['entity_type'] ?? 'node');
    $bundle = $form_state->getValue('bundle', $config['bundle'] ?? 'node');

    $form['entity_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity Type'),
      '#options' => $this->queryHelper->getEntityTypes(),
      '#default_value' => $form_state->getValue('entity_type', $config['entity_type'] ?? 'node') ?? NULL,
      '#ajax' => [
        'callback' => [$this, 'updateDependentFields'],
        'wrapper' => 'dynamic-content-wrapper',
        'event' => 'change',
        'effect' => 'fade',
      ],
      '#parents' => ['entity_type'],
      '#attributes' => [
        'style' => 'width: 300px;',
      ],
    ];

    $form['wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'dynamic-content-wrapper'],
    ];

    foreach ($this->contentBlockOptionManager->getDefinitions() as $plugin_id => $plugin_definition) {
      $plugin = $this->contentBlockOptionManager->createInstance($plugin_id, [
        'entity_type' => $entity_type,
        'bundle' => $bundle,
      ]);
      $default_value = $form_state->getValue($plugin_id, $config[$plugin_id] ?? NULL);
      $form['wrapper'][$plugin_id] = $plugin->getFormField($default_value);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['updateDependentFields'];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    $config = $this->getConfiguration();

    $entity_type = $config['entity_type'] ?? 'node';

    $this->queryHelper->setQuery($entity_type);
    foreach ($this->contentBlockOptionManager->getDefinitions() as $plugin_id => $plugin_definition) {
      $plugin = $this->contentBlockOptionManager->createInstance($plugin_id, [
        'entity_type' => $entity_type,
      ]);
      $plugin->addConditionToQuery($this->queryHelper->query, $config[$plugin_id]);
    }

    $entities = $this->queryHelper->getResults($entity_type);

    return $this->layoutRender->buildLayout($entities, $this->getConfiguration());
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    // Ensure we get the complete form state.
    if ($form_state instanceof SubformStateInterface) {
      $form_state = $form_state->getCompleteFormState();
    }

    $params = ['entity_type' => $form_state->getValue('entity_type')];
    foreach ($this->contentBlockOptionManager->getDefinitions() as $plugin_id => $plugin_definition) {
      $value = $form_state->getValue(['wrapper', $plugin_id]);
      $params[$plugin_id] = $value !== NULL ? $value : $this->getConfiguration()[$plugin_id] ?? NULL;
    }

    $this->setConfiguration($params);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $config_cache_tag = 'config:block.block.' . $this->getPluginId();
    return Cache::mergeTags(parent::getCacheTags(), ['node_list', 'taxonomy_term_list', $config_cache_tag]);
  }

  /**
   * {@inheritdoc}
   */
  public function blockAccess(AccountInterface $account) {

    if ($account->hasPermission('view content block')) {
      return AccessResult::allowed();
    }

    return AccessResult::forbidden();
  }

}
