<?php

namespace Drupal\content_block\Plugin\ContentBlockOption;

use Drupal\content_block\Plugin\ContentBlockOptionBase;
use Drupal\Core\Entity\Query\QueryInterface;

/**
 * Provides a ContentBlockOption plugin for entity bundles.
 *
 * @ContentBlockOption(
 *   id = "bundle_content_block_option",
 *   label = @Translation("Bundle Content Block Option"),
 *   description = @Translation("Applies conditions for entities bundles."),
 *   weight = 1
 * )
 */
class BundleContentBlockOption extends ContentBlockOptionBase {

  /**
   * Gets the form field for the option.
   *
   * @param mixed $default_value
   *   The default value for the form field.
   *
   * @return array
   *   The form field.
   */
  public function getFormField($default_value = NULL): array {
    return [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#options' => $this->getBundles($this->configuration['entity_type']),
      '#default_value' => $default_value ?? NULL,
      '#parents' => ['wrapper', $this->getPluginId()],
      '#attributes' => [
        'style' => 'width: 300px;',
      ],
    ];
  }

  /**
   * Adds a condition to the query.
   *
   * @param \Drupal\Core\Entity\Query\QueryInterface $query
   *   The entity query to modify.
   * @param array $param
   *   The parameters for the block.
   */
  public function addConditionToQuery(QueryInterface &$query, string $param): void {

    $entity_type_def = $this->entityTypeManager->getDefinition($this->configuration['entity_type']);

    $type = $entity_type_def->getKey('bundle');
    if (!empty($entity_type_def->getKey('bundle'))) {
      $query->condition($type, $param);
    }
  }

  /**
   * Gets available bundles for the selected entity type.
   *
   * @return array
   *   The entity bundle options.
   */
  private function getBundles($entity_type): array {
    $options = [];
    $entity_type_def = $this->entityTypeManager->getDefinition($entity_type);

    if ($bundle_entity_type = $entity_type_def->get('bundle_entity_type')) {
      $bundles = $this->entityTypeManager->getStorage($bundle_entity_type)->loadMultiple();
      foreach ($bundles as $id => $bundle) {
        $options[$id] = $bundle->label();
      }
    }
    else {
      $options[$entity_type] = $entity_type_def->get('label');
    }

    return $options;
  }

}
