<?php

namespace Drupal\content_block\Plugin\ContentBlockOption;

use Drupal\content_block\Plugin\ContentBlockOptionBase;
use Drupal\Core\Entity\Query\QueryInterface;

/**
 * Provides a ContentBlockOption plugin for entity view modes.
 *
 * @ContentBlockOption(
 *   id = "view_mode_content_block_option",
 *   label = @Translation("View mode Content Block Option"),
 *   description = @Translation("Applies conditions for entities view mode."),
 *   weight = 2
 * )
 */
class ViewModeContentBlockOption extends ContentBlockOptionBase {

  /**
   * Gets the form field for the option.
   *
   * @param mixed $default_value
   *   The default value for the form field.
   *
   * @return array
   *   The form field.
   */
  public function getFormField($default_value = NULL): array {
    return [
      '#type' => 'select',
      '#title' => $this->t('View Mode'),
      '#options' => $this->getViewModes($this->configuration['bundle']),
      '#default_value' => $default_value ?? NULL,
      '#parents' => ['wrapper', $this->getPluginId()],
      '#attributes' => [
        'style' => 'width: 300px;',
      ],
    ];
  }

  /**
   * Adds a condition to the query.
   *
   * @param \Drupal\Core\Entity\Query\QueryInterface $query
   *   The entity query to modify.
   * @param array $param
   *   The parameters for the block.
   */
  public function addConditionToQuery(QueryInterface &$query, string $param): void {}

  /**
   * Gets available view modes for a given entity type.
   *
   * @return array
   *   The entity type options.
   */
  public function getViewModes($bundle = NULL): array {
    // Get view modes for a specific entity type and a specific bundle.
    $view_modes = $this->entityTypeManager
      ->getStorage('entity_view_mode')
      ->loadByProperties(['targetEntityType' => $this->configuration['entity_type']]);

    $options = [];
    foreach ($view_modes as $view_mode) {
      $view_mode_def = $view_mode->toArray();
      if (empty($bundle) || empty($view_mode_def['dependencies']['config']) || in_array($this->configuration['entity_type'] . ".$bundle", $view_mode_def['dependencies']['config'])) {
        $view_mode_name = explode('.', $view_mode->id())[1];
        $options[$view_mode_name] = $view_mode->label();
      }
    }
    if (empty($options)) {
      $default_view_mode = $this->entityTypeManager
        ->getStorage('entity_view_mode')
        ->load('default');

      $options['default'] = $this->t('Default');
      if ($default_view_mode) {
        $options['default'] = $default_view_mode->label();
      }
    }
    return $options;
  }

}
