<?php

namespace Drupal\content_block;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Provides dynamic query helper.
 */
class QueryHelper {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The SqlContentEntityStorage.
   *
   * @var \Drupal\Core\Entity\Sql\SqlContentEntityStorage
   */
  protected $storage;

  /**
   * The Query.
   *
   * @var \Drupal\Core\Entity\Query\Sql\Query
   */
  public $query;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new DynamicQueryHelper object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Set the query.
   *
   * @param string $entity_type
   *   The entity type.
   */
  public function setQuery($entity_type): void {
    $this->storage = $this->entityTypeManager->getStorage($entity_type);
    $this->query = $this->storage->getQuery()->accessCheck(TRUE);
  }

  /**
   * Gets the query results.
   *
   * @return array
   *   The query results.
   */
  public function getResults($entity_type): array {
    $entity_ids = $this->query->execute();
    if (!$entity_ids) {
      return [];
    }

    $storage = $this->entityTypeManager->getStorage($entity_type);
    return $storage->loadMultiple($entity_ids);
  }

  /**
   * Get the entity type options.
   *
   * @return array
   *   The entity type options.
   */
  public function getEntityTypes(): array {

    $options = [];

    $config = $this->configFactory->get('content_block.settings');
    $entitiesClassesToIgnore = $config->get('entities_classes_to_ignore') ?? [];

    foreach ($this->entityTypeManager->getDefinitions() as $id => $definition) {
      // We check the Entity class is a subclass of ContentEntityInterface
      // and different from those ignored in content_block.settings.
      if (
        is_subclass_of($definition->getClass(), 'Drupal\Core\Entity\ContentEntityInterface') &&
         !in_array($definition->getClass(), $entitiesClassesToIgnore)
      ) {
        $options[$id] = $definition->getLabel();
      }
    }
    return $options;
  }

}
